/******************************************************************************
* LPREAL_D_PM.G
* Maximizes posterior density of DSGE estimation
*
* upadted: 04/23/200605:51 PM  04-Aug-2008
******************************************************************************/
new;

/** set seed **/
/** random number generator used in optimization procedure **/
/* rndseed 1234; */


/** initial value scalar**/
cc = 0;
/** set optimization routine environment **/
nit  = 100;        /** maximum number of iteration **/
crit = 1E-8;        /** tolerance **/

/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
__output = 1;       /** 1 for verbose of CSMIN**/
__graph  = 1;       /** 1 for graph mode **/
_verbose = 2;       /** 0 for silence, 1 for compact, 2 for detailed**/
_fcmptol = 1E-10;

/** the following 3 lines are model dependent **/
#include bgg_spec.g;
#include bgg_mod.src;
#include bgg_para.src;

#include pathspec.g;
#include evalmod.g;

/** path configuration **/
   #ifos2win
        postpath = postpath $+ "\\d\\" ;
        resupath = resupath $+ "\\d\\" ;
   #endif 
  
   #ifunix 
        postpath = postpath $+ "/d/" ;
        resupath = resupath $+ "/d/" ;
   #endif

chkdir = chk_dir(postpath);
chkdir = chk_dir(resupath);

/** import prior specification **/

priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/** start log file **/
   #ifos2win
        outstr = resupath $+ "\\pm.log";
   #endif 
  
   #ifunix 
        outstr = resupath $+ "/pm.log";
   #endif

output file = ^outstr reset;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

/********************************************************
* starting value for maximization
********************************************************/

/** starting value for maximization  **/
   #ifos2win
        pmfile = postpath $+ "\\pm.dat";
   #endif 
  
   #ifunix 
        pmfile = postpath $+ "/pm.dat";
   #endif

if exist(pmfile);
   open fhpara = ^pmfile for read;
   para = readr(fhpara,1)';
   closeall fhpara;
else;
   para = pmsv(msel,datasel);
endif;
npara = rows(para);


para  = para.*pmaskinv + pfix.*pmask;


/** parameter specification: transformation **/
_trspec = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;


/********************************************************
* Maximize posterior density
********************************************************/
#include bgg_p_pm.g;


{lnpY,retcode,obserror,obsvar} = evalmod_one(paraest,YY,msel,_tstar_1, _tstar_2);
retcode;

/********************************************************
* Compute Residuals
********************************************************/

/* moments */

    resid1 = obserror;
    resid1 = resid1[2:rows(resid1),.];

    resid1_m1 = meanc(resid1)';
    resid1_m2 = meanc( (resid1 - resid1_m1)^2 )';
    resid1_m3 = meanc( (resid1 - resid1_m1)^3 )';
    resid1_m4 = meanc( (resid1 - resid1_m1)^4 )';

    resid2 = obserror./sqrt(obsvar);
    resid2 = resid2[2:rows(resid2),.];
    resid2_m1 = meanc(resid1)';
    resid2_m2 = meanc( (resid2 - resid2_m1)^2 )';
    resid2_m3 = meanc( (resid2 - resid2_m1)^3 )';
    resid2_m4 = meanc( (resid2 - resid2_m1)^4 )';

    ti = ti[2:rows(ti)];

    nresid = rows(resid1);


/* autocorrelation */

    resid1_ar = zeros(1,ny);
    resid2_ar = zeros(1,ny);

    ind = 1;
    do until ind > ny;
       resid1_ar[1,ind] = inv(resid1[1:nresid-1,ind]'*resid1[1:nresid-1,ind])
                                *resid1[1:nresid-1,ind]'*resid1[2:nresid,ind];
       resid2_ar[1,ind] = inv(resid2[1:nresid-1,ind]'*resid2[1:nresid-1,ind])
                                *resid2[1:nresid-1,ind]'*resid2[2:nresid,ind];
       ind = ind+1;
    endo;


/** print on screen **/
if _verbose;
    "Raw Residuals";
    "-------------";
    "Mean:" resid1_m1;
    "Stdd:" sqrt(resid1_m2);
    "Skew:" resid1_m3./resid1_m2^(3/2);
    "Kurt:" resid1_m4./(3*resid1_m2^2);
    "Corr:" resid1_ar;
    "";
    "Standardized Residuals";
    "----------------------";
    "Mean:" resid2_m1;
    "Stdd:" sqrt(resid2_m2);
    "Skew:" resid2_m3./resid2_m2^(3/2);
    "Kurt:" resid2_m4./(3*resid2_m2^2);
    "Corr:" resid2_ar;
    "";

	if __graph;
		/** titlestr_ is defined in ****_spec.g **/
		ystr     =  "Raw Residual " $|
						"Stdd Residual";

		graphset;
	   fonts("microb");
	   _ptitlht = 0.4;
	   _paxht   = 0.4;
	   _pnumht  = 0.4;
	   _ptek    = resupath $+ "\\pmresid.tkf";

	   begwind;
	   window(2,cols(YY),0);

		ser_ind = 1;
	   do until ser_ind > cols(YY);

			ylabel(ystr[1]);
   	      title(titlestr_[ser_ind]);
			xy(ti,resid1[.,ser_ind]);

	      ser_ind = ser_ind+1;
		   nextwind;

	   endo;

		ser_ind = 1;
		do until ser_ind > cols(YY);

	      ylabel(ystr[2]);
		   title(titlestr_[ser_ind]);
			xy(ti,resid2[.,ser_ind]);

	      ser_ind = ser_ind+1;
		   nextwind;

		endo;

		endwind;
	endif;	/** graph__ **/

endif;  /** _verbose **/


/** close log file **/
output file = ^outstr off;


/********************************************************
* PROCEDURE: OBJFCN
* defines the objective function to minimize:
* the negative of log posterior
*********************************************************/
proc (1) = objfcn(para);
local lnpY,lnprio,obserror,obsvar,modelpara,retcode, prueba,k,kk;

/** likelihood **/
modelpara = trans(para);
/*modelpara;
k=keyw;*/
modelpara = modelpara.*pmaskinv + pfix.*pmask;

/*{lnpY,retcode,obserror,obsvar} = evalmod(modelpara,YY,msel);*/
{lnpY,retcode,obserror,obsvar} = evalmod_one(modelpara,YY,msel,_tstar_1, _tstar_2);
/*lnpy;
kk=keyw;
*/

if ismiss(lnpY);
   lnpY = -1E8;
endif;

/** Evaluate the Prior distribution **/
lnprio = priodens(modelpara, pmean, pstdd, pshape);

/** minimize the negative of log posterior **/
retp(real(-lnpY-lnprio));
endp;
