/*************************************************************************
** filename:    dsge_filt.g
** description: The program filters out neutral technology, investment-specific, 
                and financial shocks 
*************************************************************************/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include bgg_spec.g;

//#include bgg_mod_sim.src;
//#include bgg_para_sim.src;
#include bgg_mod.src;
#include bgg_para.src;
#include pathspec.g;

/******************************************************** 
** load data 
*********************************************************/

#include loaddata.g;
nobs  = rows(YY);       
T0 = 0;

/* Open files with Parameter Draws
*/
mhrun  = "3";

   #ifos2win
        lpara = parapath $+ "\\mhrun" $+ mhrun $+ "\\" $+"pa.dat";
   #endif 
  
   #ifunix 
        lpara = parapath $+ "/mhrun" $+ mhrun $+ "/" $+"pa.dat";
   #endif

open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 5000;	/* block size */
skip   = 1;		/* skip = 1: use only first draw from block */
nburn  = 50;

/* Initialize Output files
*/
   #ifos2win
        opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
   #endif 
  
   #ifunix 
        opath = resupath $+ "/mhrun" $+ mhrun $+ "/";
   #endif

chkdir = chk_dir(opath);

ozzt        = opath $+ "zzt.dat";
ozt         = opath $+ "zt.dat";
ozzetat     = opath $+ "zzetat.dat";
ozetat      = opath $+ "zetat.dat";
ommut       = opath $+ "mmut.dat";
omut        = opath $+ "mut.dat";
ollambdat   = opath $+ "llambdat.dat";
olambdat    = opath $+ "lambdat.dat";
otthetat    = opath $+ "tthetat.dat";
othetat     = opath $+ "thetat.dat";
obbt        = opath $+ "bbt.dat";
obt         = opath $+ "bt.dat";
orrt        = opath $+ "rrt.dat";
ort         = opath $+ "rt.dat";
oggamt      = opath $+ "ggamt.dat";
ogamt       = opath $+ "gamt.dat";
oggt        =    opath $+ "ggt.dat";
ogt         =    opath $+ "gt.dat";

create fhzzt        = ^ozzt with zzT, nobs-T0, 8;
create fhzt         = ^ozt with zT, nobs-T0, 8;
create fhzzetat     = ^ozzetat with zzetaT, nobs-T0, 8;
create fhzetat      = ^ozetat with zetaT, nobs-T0, 8;
create fhmmut       = ^ommut with mmuT, nobs-T0, 8;
create fhmut        = ^omut with muT, nobs-T0, 8;
create fhllambdat   = ^ollambdat with llambdaT, nobs-T0, 8;
create fhlambdat    = ^olambdat with lambdaT, nobs-T0, 8;
create fhtthetat    = ^otthetat with tthetaT, nobs-T0, 8;
create fhthetat     = ^othetat with thetaT, nobs-T0, 8;
create fhbbt        = ^obbt with bbT, nobs-T0, 8;
create fhbt         = ^obt with bT, nobs-T0, 8;
create fhrrt        = ^orrt with rrT, nobs-T0, 8;
create fhrt         = ^ort with rT, nobs-T0, 8;
create fhggamt      = ^oggamt with ggamT, nobs-T0, 8;
create fhgamt       = ^ogamt with gamT, nobs-T0, 8;
create fhggt        = ^oggt with ggT, nobs-T0, 8;     
create fhgt         = ^ogt with gT, nobs-T0, 8;  

        
/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
/*pararow = seekr(fhpara,nburn);*/

do until eofloop; 

   /* Read blocks of size nsim
   */

   parasim = readr( fhpara, nsim );

   nread = rows(parasim);
   
   if skip == 0;   
      zztsim        = zeros(nread,nobs-T0);
      ztsim         = zeros(nread,nobs-T0);
      zzetatsim     = zeros(nread,nobs-T0);
      zetatsim      = zeros(nread,nobs-T0);
      mmutsim       = zeros(nread,nobs-T0);
      mutsim        = zeros(nread,nobs-T0);
      llambdatsim   = zeros(nread,nobs-T0);
      lamdbatsim    = zeros(nread,nobs-T0);
      bbtsim        = zeros(nread,nobs-T0);
      btsim         = zeros(nread,nobs-T0);
      rrtsim        = zeros(nread,nobs-T0);
      rtsim         = zeros(nread,nobs-T0);
      ggamtsim      = zeros(nread,nobs-T0);
      gamtsim       = zeros(nread,nobs-T0);
      ggtsim        = zeros(nread, nobs-T0);
      gtsim         = zeros(nread, nobs-T0);
    else;
      zztsim        = zeros(1,nobs-T0);
      ztsim         = zeros(1,nobs-T0);
      zzetatsim     = zeros(1,nobs-T0);
      zetatsim      = zeros(1,nobs-T0);
      mmutsim       = zeros(1,nobs-T0);
      mutsim        = zeros(1,nobs-T0);
      llambdatsim   = zeros(1,nobs-T0);
      lambdatsim    = zeros(1,nobs-T0);
      tthetatsim    = zeros(1,nobs-T0);
      thetatsim     = zeros(1,nobs-T0);
      bbtsim        = zeros(1,nobs-T0);
      btsim         = zeros(1,nobs-T0);
      rrtsim         = zeros(1,nobs-T0);
      rtsim         = zeros(1,nobs-T0);
      ggamtsim      = zeros(1,nobs-T0);
      gamtsim       = zeros(1,nobs-T0);
      ggtsim        = zeros(1,nobs-T0);
      gtsim         = zeros(1,nobs-T0);
    endif;

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, conduct the filtering
      */   
/*here i need to redo the stuff by subsamples!!!!*/
     statesim = dsgefilt_one( parasim[j,.]', msel, YY, _tstar_1, _tstar_2 );

     zztsim[j,.]        = statesim[T0+1:nobs,1]';  
     ztsim[j,.]         = statesim[T0+1:nobs,2]'; 
     zzetatsim[j,.]     = statesim[T0+1:nobs,3]';    
     zetatsim[j,.]      = statesim[T0+1:nobs,4]';    
     mmutsim[j,.]       = statesim[T0+1:nobs,5]';
     mutsim[j,.]        = statesim[T0+1:nobs,6]';
     llambdatsim[j,.]   = statesim[T0+1:nobs,7]';        
     lambdatsim[j,.]    = statesim[T0+1:nobs,8]';        
     tthetatsim[j,.]    = statesim[T0+1:nobs,9]';    
     thetatsim[j,.]     = statesim[T0+1:nobs,10]';    
     bbtsim[j,.]        = statesim[T0+1:nobs,11]';    
     btsim[j,.]         = statesim[T0+1:nobs,12]';    
     ggamtsim[j,.]      = statesim[T0+1:nobs,13]';  
     gamtsim[j,.]       = statesim[T0+1:nobs,14]'; 
     rrtsim[j,.]        = statesim[T0+1:nobs,15]';  
     rtsim[j,.]         = statesim[T0+1:nobs,16]';    
     ggtsim[j,.]        = statesim[T0+1:nobs,17]';
     gtsim[j,.]         = statesim[T0+1:nobs,18]';
    
     "Iteration  " j "of" nsim;
     pause(1E-3);
/*"z";;ztsim[j,.];
"zeta";;zetatsim[j,.];
"mu";;mutsim[j,.];
k=keyw;*/
     if skip == 1;
        j = rows(parasim)+6;
     else;
        j = j+1;
     endif;
  endo;
/*mutsim'~mmutsim';
k=keyw;
gamtsim'~ggamtsim';
k=keyw;*/
  /* Write Output
  */
  wr = writer(fhzzt,zztsim);
  wr = writer(fhzt,ztsim);
  wr = writer(fhzzetat,zzetatsim);
  wr = writer(fhzetat,zetatsim);
  wr = writer(fhmmut,mmutsim);
  wr = writer(fhmut,mutsim);
  wr = writer(fhllambdat,llambdatsim);
  wr = writer(fhlambdat,lambdatsim);
  wr = writer(fhtthetat,tthetatsim);
  wr = writer(fhthetat,thetatsim);
  wr = writer(fhbbt,bbtsim);
  wr = writer(fhbt,btsim);
  wr = writer(fhrrt,rrtsim);
  wr = writer(fhrt,rtsim);
  wr = writer(fhggamt,ggamtsim);
  wr = writer(fhgamt,gamtsim);
  wr = writer(fhggt,ggtsim);
  wr = writer(fhgt,gtsim);
  
  loopct = loopct + 1;
  "Loop     " loopct;
  eofloop = eof(fhpara);

endo;

closeall fhzzt, fhzt,fhzzetat, fhzetat, fhmmut, fhmut, fhllambdat, fhlambdat, fhtthetat, fhthetat, fhbbt, fhbt,fhrt,fhggamt,fhgamt,fhgt,fhggt, fgrrt, fhpara;

#include bgg_filt.src;
