/*************************************************************************
** filename:    dsge_filt.g
** description: The program filters out neutral technology, investment-specific, 
                and financial shocks 
*************************************************************************/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/
#include bgg_spec.g;

#include bgg_mod.src;
#include bgg_para.src;
#include pathspec.g;



#include bgg_filt.src;

/******************************************************** 
** load data 
*********************************************************/

#include loaddata.g;
nobs  = rows(YY);       
T0 = 0;

/* Open files with Parameter Draws
*/
mhrun  = "1";

   #ifos2win
        lpara = parapath $+ "\\mhrun" $+ mhrun $+ "\\" $+"pa.dat";
   #endif 
  
   #ifunix 
        lpara = parapath $+ "/mhrun" $+ mhrun $+ "/" $+"pa.dat";
   #endif

open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 100000;	/* block size */
skip   = 1;		/* skip = 1: use only first draw from block */
nburn  = 50;

/* Initialize Output files
*/
   #ifos2win
        opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
   #endif 
  
   #ifunix 
        opath = resupath $+ "/mhrun" $+ mhrun $+ "/";
   #endif

chkdir = chk_dir(opath);
oyt           		= opath $+ "yt.dat";
onwt          		= opath $+ "nwt.dat";
oit           		= opath $+ "it.dat";
odeltayt      		= opath $+ "deltaayt.dat";
odeltanwt     		= opath $+ "deltanwt.dat";
odeltait      		= opath $+ "deltait.dat";

oxpt          		= opath $+ "lxp.dat";

olevelyt            = opath $+ "levelyt.dat";
olevelnwt           = opath $+ "levelnwt.dat";
olevelit            = opath $+ "levelit.dat";

ocycleyt            = opath $+ "cycleyt.dat";
ocyclenwt           = opath $+ "cyclenwt.dat";
ocycleit            = opath $+ "cycleit.dat";

create fhyt            = ^oyt with yT, nobs-T0, 8;
create fhnwt           = ^onwt with nwT, nobs-T0, 8;
create fhit            = ^oit with iT, nobs-T0, 8;
create fhdeltayt       = ^odeltayt with deltayT, nobs-T0, 8;
create fhdeltanwt      = ^odeltanwt with deltanwT, nobs-T0, 8;
create fhdeltait       = ^odeltait with deltaiT, nobs-T0, 8;

create fhlevelyt       = ^olevelyt with levelyT, nobs-T0, 8;
create fhlevelnwt      = ^olevelnwt with levelnwT, nobs-T0, 8;
create fhlevelit       = ^olevelit with leveliT, nobs-T0, 8;

create fhcycleyt       = ^ocycleyt with cycleyT, nobs-T0, 8;
create fhcyclenwt      = ^ocyclenwt with cyclewT, nobs-T0, 8;
create fhcycleit       = ^ocycleit with cycleiT, nobs-T0, 8;


create fhxpt          = ^oxpt with xpT, nobs-T0, 8;






/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
/*pararow = seekr(fhpara,nburn);*/

do until eofloop; 

   /* Read blocks of size nsim
   */

   parasim = readr( fhpara, nsim );

   nread = rows(parasim);
   
   if skip == 0;   
      ytsim  = zeros(nread,nobs-T0);
      nwtsim = zeros(nread,nobs-T0);
      itsim = zeros(nread,nobs-T0);
      deltaytsim  = zeros(nread,nobs-T0);
      deltanwtsim = zeros(nread,nobs-T0);
      deltaitsim = zeros(nread,nobs-T0);

	  xptsim = zeros(nread,nobs-T0);

	  levelytsim  = zeros(nread,nobs-T0);
      levelnwtsim = zeros(nread,nobs-T0);
      levelitsim = zeros(nread,nobs-T0);
      cycleytsim  = zeros(nread,nobs-T0);
      cyclenwtsim = zeros(nread,nobs-T0);
      cycleitsim = zeros(nread,nobs-T0);
   else;
      ytsim  = zeros(1,nobs-T0);
      nwtsim = zeros(1,nobs-T0);
      itsim = zeros(1,nobs-T0);
      deltaytsim  = zeros(1,nobs-T0);
      deltanwtsim = zeros(1,nobs-T0);
      deltaitsim = zeros(1,nobs-T0);

	  xptsim = zeros(1,nobs-T0);
	  levelytsim  = zeros(1,nobs-T0);
      levelnwtsim = zeros(1,nobs-T0);
      levelitsim = zeros(1,nobs-T0);  

	  cycleytsim  = zeros(1,nobs-T0);
      cyclenwtsim = zeros(1,nobs-T0);
      cycleitsim = zeros(1,nobs-T0); 	  
   endif;


   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, conduct the filtering
      */   
/*here i need to redo the stuff by subsamples!!!!*/
     statesim = dsgefilt_states_one( parasim[j,.]', msel, YY, _tstar_1, _tstar_2 );
 
     ytsim[j,.]  = statesim[T0+1:nobs,1]'; 
     nwtsim[j,.] = statesim[T0+1:nobs,2]';
     itsim[j,.]  = statesim[T0+1:nobs,3]';
     deltaytsim[j,.]  = statesim[T0+1:nobs,4]'; 
     deltanwtsim[j,.] = statesim[T0+1:nobs,5]';
     deltaitsim[j,.]  = statesim[T0+1:nobs,6]';
     xptsim[j,.]  = statesim[T0+1:nobs,7]';

	 levelytsim[j,.]  = statesim[T0+1:nobs,8]'; 
     levelnwtsim[j,.] = statesim[T0+1:nobs,9]';
     levelitsim[j,.]  = statesim[T0+1:nobs,10]';
     cycleytsim[j,.]  = statesim[T0+1:nobs,11]'; 
     cyclenwtsim[j,.] = statesim[T0+1:nobs,12]';
     cycleitsim[j,.]  = statesim[T0+1:nobs,13]';

	 

	 
     "Iteration  " j "of" nsim;
     pause(1E-3);

     if skip == 1;
        j = rows(parasim)+6;
     else;
        j = j+1;
     endif;
  endo;

  /* Write Output
  */
  wr = writer(fhyt,ytsim);
  wr = writer(fhnwt,nwtsim);
  wr = writer(fhit,itsim);

  wr = writer(fhdeltayt,deltaytsim);
  wr = writer(fhdeltanwt,deltanwtsim);
  wr = writer(fhdeltait,deltaitsim);
  wr = writer(fhxpt,xptsim);
  wr = writer(fhlevelyt,levelytsim);
  wr = writer(fhlevelnwt,levelnwtsim);
  wr = writer(fhlevelit,levelitsim);

  wr = writer(fhcycleyt,cycleytsim);
  wr = writer(fhcyclenwt,cyclenwtsim);
  wr = writer(fhcycleit,cycleitsim);
  
  loopct = loopct + 1;
  "Loop     " loopct;
  eofloop = eof(fhpara);

endo;

closeall fhyt, fhnwt, fhit, fhdeltayt, fhdeltait, fhdeltanwt, fhxpt,fhlevelyt, fhlevelnwt, fhlevelit, 

fhcycleyt, fhcyclenwt, fhcycleit;

