/******************************************************************************
* NKMP_MH.G
* generates draws from the posterior distribution
* via the random walk Metropolis-Hastings algorithm
*
* upadted:  06/16/2005 
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/
#include bgg_spec.g;
#include bgg_mod.src;
#include bgg_para.src;


#include pathspec.g;
#include evalmod.g;
/********************************************************
* Configure the Metropolis algorithm
********************************************************/

mhrun = "1";

nblocks = 10;
nsim    = 2500;//1000;//5000;

cc0     = 1;

cc=.18;//.13;//.18;
/* 
** use cc = 0.25 for s5t1
*/

appendmode = 0;


/** start log file **/
   #ifos2win
        outstr = resupath $+ "\\mh.log";
   #endif 
  
   #ifunix 
        outstr = resupath $+ "/mh.log";
   #endif

output file = ^outstr reset;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/** parameter specification: transformation and bounds **/
_trspec      = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;
_bounds      = _trspec[.,2 3];


/********************************************************
* load Posterior Mode Estimates
********************************************************/

   #ifos2win
        lpara = postpath $+"\\d" $+"\\pm" ;
        lretc = postpath  $+"\\d"$+ "\\pmret" ;
   #endif 
  
   #ifunix 
        lpara = postpath $+"/d" $+"/pm" ;
        lretc = postpath  $+"/d"$+ "/pmret" ;
   #endif


/** check posterior mode results **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run posterior maximization program first.";
    "";
    end;
endif;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para  = para';          /** paraest was saved as row vector in _pm.g **/

npara = rows(para);

open fhretc = ^lretc for read;
_csretcode = readr(fhretc,1);
closeall fhretc;

/********************************************************
* load multiplication matrix for parameters
********************************************************/
   #ifos2win
        lmult = postpath  $+"\\d" $+ "\\mult";       /** sigmult **/
        lhess = postpath  $+"\\d" $+ "\\hess";     /** HHM **/
   #endif 
  
   #ifunix 
        lmult = postpath  $+"/d" $+ "/mult";       /** sigmult **/
        lhess = postpath  $+"/d" $+ "/hess";     /** HHM **/
   #endif




/** check Hessian results **/
if not(exist(lmult $+ ".dat"));
    "";
    "Please run Hessian finding program first.";
    "";
    end;
endif;

/** open **/
open fhmult = ^lmult for read;
SIGSCALE    = readr( fhmult,npara );
closeall fhmult;

open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;


/*sigscale[1,2]=-0.99;
sigscale[1,68]=0.99;
sigscale[2,9]=-0.99;
sigscale[2,69]=0.99;
sigscale[9,69]=0.99;
sigscale[68,69]=0.99;
*/

/*
sigscale[8,16]=0.99;
sigscale[8,57]=0.99;
sigscale[8,58]=0.99;
sigscale[11,58]=0.99;
sigscale[11,70]=0.99;
sigscale[54,58]=0.99;
sigscale[56,58]=0.99;
sigscale[57,58]=0.99;
sigscale[57,73]=0.99;
*/

/*sigscale[27,11]=0.99;
sigscale[7,54]=0.99;
sigscale[7,57]=0.99;
sigscale[10,70]=0.99;
sigscale[16,58]=-0.99;*/

SIGPROPdim = sumc(pmaskinv);

/** inverse by Singular Value Decomposition (SVD) **/
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

/** log determinant for nonsingular part **/
i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;


/********************************************************
* Initialize Output files
********************************************************/
   #ifos2win
        opath   = parapath $+ "\\mhrun" $+ mhrun;
   #endif 
  
   #ifunix 
        opath   = parapath $+ "/mhrun" $+ mhrun;
   #endif

chkdir  = chk_dir(opath);

/** print specification **/
if _verbose;

    "";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $mprior;
    "MHrun Index      " $mhrun;
    "Scaling of Hess  " cc;
    "";
    "===================================";
    "";

    "";
   outmat = ("Parameter"  $~ "Estimate") $|
            ( (para_names') $~ ftocv( para,4,4) );

   $outmat;
   
   {lnpost0, lnpy0} = fcn2(para);

    "";
    "Prior*Likelihood at Mode:" lnpost0;
    "Likelihood at Mode      :" lnpy0;
    "";
    "Press Key to Continue";
    k = keyw;
    "";

endif;      /** _verbose **/


statname = { POST LIKE PROP PROPPOST REJECT };

   #ifos2win
        ostat = opath $+ "\\st" $+".dat" ;
        opara = opath $+ "\\pa"$+".dat";
   #endif 
  
   #ifunix 
        ostat = opath $+ "/st" $+".dat" ;
        opara = opath $+ "/pa"$+".dat";
   #endif


/** check if append is possible **/
if not(exist(opara $+ ".dat"));
    appendmode=0;
endif;

if appendmode ne 1;

  /*******************************
  ** create new output files
  *******************************/

  "-----------------------------------------------------------";
  "Creating new output files";
  "Press Key to Continue";
  "-----------------------------------------------------------";
  k = keyw;
  "";

  create fhstat = ^ostat with ^statname, 5, 4;
  create fhpara = ^opara with ^para_names, npara, 4;

else;

  /*******************************
  ** open existing output files
  *******************************/

  "-----------------------------------------------------------";
  "Opening Existing Output Files";
  "Press Key to Continue";
  "-----------------------------------------------------------";

  open fhpara = ^opara for update;
  open fhstat = ^ostat for update;

  rowind = seekr(fhpara,rowsf(fhpara));
  rowind = seekr(fhstat,rowsf(fhpara));

  para    = readr(fhpara,1)';
  statsim = readr(fhstat,1);

  "Number of draws in File" rowind;
  "";

  rowind = seekr(fhpara,0);
  rowind = seekr(fhstat,0);

  cc0 = 0;

  "Parameter   | Last Draw ";
  outmat = para_names'~para;
  let mask[1,2] = 0 1;
  let fmt[2,3] =
               "-*.*s " 10 6
               "*.*lf " 10 4;
  d = printfm(outmat,mask,fmt);

  "";
  "Press Key to Continue";
  "-----------------------------------------------------------";
  k = keyw;
  
endif;

/********************************************************
* Metropolis-Hastings algorithm
********************************************************/
cc0=0.5;


/** initialize PARANEW **/
/** trembles the posterior mode, and posterior is large enough **/
valid = 0;
do until valid;
   paranew = para + cc0*(sigscale*rndn(npara,1));
   paranew = paranew.*pmaskinv + pfix.*pmask;

   {postnew, likenew} = fcn2(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -10000;
      valid = 1;
   endif;
endo;

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   if _verbose;
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);
   endif;

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if (iblock == 1) and (appendmode ne 1);
      /** save initial value as 1st draw in 1st block **/
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;
   elseif (iblock == 1) and (appendmode == 1);
      j = 1;
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;
   else;
      /** don't save initial values **/
      j=1;
   endif;

   /** with-in block loop **/
    do until j > nsim;

      /** propose **/
      paranew = paraold + cc*(sigscale*rndn(npara,1));
      paranew = paranew.*pmaskinv + pfix.*pmask;
       /* "paranew";
        paranew';
       k=keyw; */
      /** evaluate at the candidate **/
      {postnew, likenew} = fcn2(paranew);
      propdens = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                 -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
      propsim[j,1] = propdens;
      proppostsim[j,1] = postnew;
      /*"postnew";
      postnew;
      "postold";
      postold;
      k=keyw;*/
      
      /** accept/reject rule  **/
      r = minc(1 | exp( postnew - postold));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          postsim[j,1] = postnew;
          likesim[j,1] = likenew;
          parasim[j,.] = paranew';

          /** updates **/
          paraold = paranew;
          postold = postnew;
          likeold = likenew;

      else;

          /** reject proposed jump **/
          likesim[j,1] = likeold;
          postsim[j,1] = postold;
          parasim[j,.] = paraold';
          rej[j] = 1;

      endif;

      /**
      "Block" iblock "of" nblocks;
      "Simulation step" j "of" nsim;
      "Proposal Dens  " propsim[j,1];
      "Parameters     "; parasim[j,.];
      **/

        if _verbose;
            if fmod(j,100)==0;".";; pause(1e-20); endif;
        endif;

      j = j+1;
    endo;               /** with-in block  **/

   if _verbose;
       " ";
       "Rejection: perct " meanc(rej);
       "Likelihood       " meanc(likesim);
       "Posterior        " meanc(postsim);
       "Parameters       "; meanc(parasim)';
       "";
       pause(1e-20);
   endif;  /** verbose **/

   /** set finish time **/
   tfin = date;

   if _verbose;
       "Elapsed Time"; etstr(ethsec(tstart,tfin));
       "";
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;           /** block **/

closeall fhstat, fhpara;

#include mhdd.g;

/** close log file **/
output file = ^outstr off;


/****************************************************
* PROCEDURES: FCN2
* evaluates posterior density given parameter bounds
****************************************************/
proc (2) = fcn2(para);
local lnpY, lnprio, obsmean, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;

    parabd_ind1 = para .> _bounds[.,1];         /** lower bounds **/
    parabd_ind2 = para .< _bounds[.,2];         /** upper bounds **/
    parabd_ind1 = delif(parabd_ind1,pmask);
    parabd_ind2 = delif(parabd_ind2,pmask);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);   /** in bounds **/

        {lnpY,retcode,obsmean,obsvar} = evalmod_one(para,YY,msel,_tstar_1, _tstar_2);
        lnpy    = real(lnpy);
        lnprio  = priodens( para, pmean, pstdd, pshape);
        lnpost  = lnpy + real(lnprio);

    else;                                                           /** out of bounds **/

        lnpost = -1E6;
        lnpy   = -1E20;

    endif;

retp(lnpost,lnpy);
endp;
