/******************************************************************************
* LPREAL_P_PM.G
* Maximizes posterior density of DSGE estimation
*
* upadted: 05:59 PM  04-Aug-2008
******************************************************************************/

/********************************************************
* Calculate posterior density at starting values
* (verbose mode only)
********************************************************/
if _verbose;
    "Model              " $lmodel;
    "Prior              " $lprior;
    "Dataset            " $dataselstr;
    
    "===================================";
    "";
    "Prior Density at Starting Value:";; priodens(para,pmean,pstdd,pshape);
    lnpy = -objfcn(invtrans(para));
    "Posterior at Starting Value:    ";; lnpy;
    "Press key to continue";
    k = keyw;
    "";

    if not(__output);
    "";"";
    "Optimization in progress..";
    "--------------------------";
    endif;
endif;    /**   _verbose **/


/********************************************************
* Maximize the posterior density
********************************************************/

/** initial values: x0 and H0 **/
/*cc = 0.1;this was comment out*/
x0 = invtrans(para) + cc* ( rndn(npara,1) );
H0 = eye(npara)*1E-4;

/** set optimization routine environment **/
nit  = 1000;        /** maximum number of iteration **/
crit = 1E-6;        /** tolerance **/

/** Use personal optimization procedure **/
/** translated from Chris Sims' CSMIN package **/
{fh,xh,g,H,itct,fcount,_csretcode} = csminwel(x0,H0,crit,nit,&objfcn);
/*"fh";
fh;
"xh";
xh';
k=keyw;
*/
/********************************************************
* save parameter estimates and return code
********************************************************/

/** max to model parameters **/
paraest = trans(xh);
paraest = real(paraest);
paraest = paraest.*pmaskinv + pfix.*pmask;


/** output file name **/

   #ifos2win
        oparaest = postpath $+ "\\pm" $+ ".dat";
        opostval = postpath $+ "\\pmval" $+ ".dat";
        oretcode = postpath $+ "\\pmret" $+ ".dat";
   #endif 
  
   #ifunix 
        oparaest = postpath $+ "/pm" $+ ".dat";
        opostval = postpath $+ "/pmval" $+ ".dat";
        oretcode = postpath $+ "/pmret" $+ ".dat";
   #endif


/** make output file **/
create fhparaest = ^oparaest with ^para_names, npara, 8;
wr = writer(fhparaest,paraest');
closeall fhparaest;

create fhretcode = ^oretcode with RETCODE, 1, 2;
wr = writer(fhretcode,_csretcode);
closeall fhretcode;


/********************************************************
* Estimation Output
********************************************************/
lnpY   = -objfcn(invtrans(paraest));
lnprio = priodens( paraest, pmean, pstdd, pshape);

/** posterior values **/
postnames = { LNPOST LNPY LNPRIO };
create fhpostval = ^opostval with ^postnames, 3, 8;
wr = writer(fhpostval,lnpy~(lnpy-lnprio)~lnprio);
closeall fhparaest;

/* Compute gradient in terms of model parameters */
gradmod = g./ diag(gradp(&invtrans,paraest));

if _verbose;
    "";
    "Model              " $lmodel;
    "Prior              " $lprior;
    "Dataset            " $dataselstr;
   
    "===================================";;
    "Posterior Mode     " lnpy;
    "Likelihood at Mode " lnpy-lnprio;
    "Prior at Mode      " lnprio;
    "";
    "Parameter | Estimate | Start-V | Prior Mean | Prior Stdd | Gradient ";
    outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
    let mask[1,6] = 0 1 1 1 1 1;
    let fmt[6,3] = "-*.*s" 10 6
                   "*.*lf"  9 4
                   "*.*lf" 11 4
                   "*.*lf" 11 4
                   "*.*lf" 13 4
                   "*.*lf" 12 4;
    d = printfm(outmat,mask,fmt);
    "";
    "Return code:        ";; _csretcode;
    "";
    
endif;      /** _verbose **/
