/********************************************************************
* EVALMOD.G 
* Evaluates the likelihood of DSGE model and contains the following procedures:
*
* {loglh,retcode,obserror,obsvar} = evalmod_one(para,YY,msel,t_star_1,t_star_2);
* 
********************************************************************/

/********************************************************************
** Likelihood Evaluation using Kalman Filter
**
** {loglh,retcode,obserror,obsvar} = evalmod_one(para,YY,msel);
**
** #include ****_mod.src,  to run this procedure
**
** Last updated: 02:19 PM  19-Mar-2009
********************************************************************/
proc (4) = evalmod_one(para,YY,msel,t_star_1,t_star_2);

local loglh,retcode,retcode2,obserror,obsvar,loglhzero,
      nobs,ny,nstate,nvar,nz,nep,para3,
      para1,para2,ssteady1,ssteady2,ssteady3,retcode3,
      TT,QQ,RR,HH,DD,ZZ,VV,RC,JJ,TT3,QQ3,RR3,HH3,DD3,ZZ3,VV3,RC3,JJ3,
      TT2,QQ2,RR2,HH2,DD2,ZZ2,VV2,RC2,JJ2,ssteady,
      t,At,Pt,At1,Pt1,Ft,alphahat,Phat,nut,yhat,
      Tqz, Sqz, Qqz, Zqz, lambdaqz, RRqz, nstate_rr, wtoalp, alptow,i,k34;

/** initialize **/
nobs = rows(YY);
ny   = cols(YY);

loglhzero = -1E8;

loglh     = 0;
retcode   = 0;
retcode2   = 0;
retcode3   = 0;
obserror  = zeros(nobs,ny);
obsvar    = zeros(nobs,ny);

para1     = para[1:3]|para[6:16]|para[19]|para[22:30]|para[33]|para[36]|para[39]|para[42]|para[45]|para[48]|para[51]|para[54:62]|para[63]|para[66:69]|para[70:72];
para2     = para[1:2]|para[4]|para[6:15]|para[17]|para[20]|para[22:29]|para[31]|para[34]|para[37]|para[40]|para[43]|para[46]|para[49]|para[52]|para[54:62]|para[64]|para[66:69]|para[70:72];
para3     = para[1:2]|para[5:15]|para[18]|para[21:29]|para[32]|para[35]|para[38]|para[41]|para[44]|para[47]|para[50]|para[53:62]|para[65]|para[66:69]|para[70:72];

/**********************************************************
**      System matrices for State-space
**********************************************************/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,JJ}           =sysmat(para1,msel);
{TT2,QQ2,RR2,HH2,DD2,ZZ2,VV2,RC2,JJ2}  =sysmat(para2,msel);
{TT3,QQ3,RR3,HH3,DD3,ZZ3,VV3,RC3,JJ3}  =sysmat(para3,msel);
{ssteady1,ssteady2,ssteady3,ssteady}            =sysmat_ssteady(para,msel);
//"JJ";JJ~JJ2~JJ3;
//"====";
//k34=keyw;
//"RC";
//RC'~RC2'~RC3';
//"";
//"RC2";
//RC2';
//"RC3";
//RC3';
//k34=keyw;
/*if para[5]>=para[4];
   loglh = loglhzero;
   para[5]=.75*para[4];
   goto eofeval;
endif;*/
/**********************************************************
** Check determinacy 
**********************************************************/
if (RC[1] == 1) AND (RC[2]==1);
   /* determinacy */
   retcode[1] = 0;

elseif (RC[1] == 1) AND (RC[2]==0) ;
   /* indeterminacy */
   retcode[1] = 1;
   loglh = loglhzero;
   goto eofeval;
  /**/       
else;
   /* no equilibrium exists, numerical problems */
   retcode[1] = RC[1];
   loglh = loglhzero;
   goto eofeval;
/**/
endif;
if (RC2[1] == 1) and (RC2[2]==1);
   /* determinacy */
   retcode2[1] = 0;

elseif (RC2[1] == 1) and (RC2[2]==0) ;
   /* indeterminacy */
   retcode2[1] = 1;
   loglh = loglhzero;
   goto eofeval;
/**/
else;
   /* no equilibrium exists, numerical problems */
   retcode2[1] = RC2[1];
  /**/ loglh = loglhzero;
   goto eofeval;

endif;
if (RC3[1] == 1) and (RC3[2]==1);
   /* determinacy */
   retcode3[1] = 0;

elseif (RC3[1] == 1) and (RC3[2]==0) ;
 /*   indeterminacy */
   retcode3[1] = 1;
  /**/ loglh = loglhzero;
   goto eofeval;

else;
   /* no equilibrium exists, numerical problems */
   retcode3[1] = RC3[1];
  /**/ loglh = loglhzero;
   goto eofeval;

endif;

/**********************************************************
**      Likelihood evaluation by Kalman filter
**********************************************************/
nstate = rows(TT);

/** initial mean and variance for the state vector **/
/*At = zeros(nstate,1);*/
At= ssteady1;
//Pt =dlyap(TT,RR*QQ*RR');

Pt = eye(cols(TT));
/** filter loop **/
t = 1;
do until t > nobs;

   At1 = At;
   Pt1 = Pt;
/*"t";t;
At1';
k34=keyw;*/
   /** Forecasting **/
   if t < t_star_1;

   /*    alphahat = TT*At1;*/
       alphahat = TT*At1 + JJ;
       Phat     = TT*Pt1*TT' + RR*QQ*RR';

       Phat     = .5*(Phat+Phat');
    
      /* yhat     = ZZ*(alphahat - ssteady1) + DD;*/
       yhat     = ZZ*alphahat;
       nut      = YY[t,.] - yhat';
       Ft       = ZZ*Phat*ZZ' + HH + ZZ*RR*VV + (ZZ*RR*VV)';
  endif;
 if t>=t_star_1 and t < t_star_2; 
      if t == t_star_1;
          alphahat = TT2*At1 + JJ2 + TT2*(ssteady2 - ssteady1);
      elseif t < t_Star_2;
           alphahat = TT2*At1 + JJ2;
      endif;
       Phat     = TT2*Pt1*TT2' + RR2*QQ2*RR2';
    
       Phat     = .5*(Phat+Phat');
    
       /*yhat     = ZZ2*(alphahat - ssteady2) + DD2;*/
       yhat     = ZZ2*alphahat;
       nut      = YY[t,.] - yhat';
       Ft       = ZZ2*Phat*ZZ2' + HH2 + ZZ2*RR2*VV2 + (ZZ2*RR2*VV2)';
endif;
 if t >= t_star_2;   
      if t == t_star_2;
            alphahat = TT3*At1 + JJ3 + TT3*(ssteady3 - ssteady2);
      elseif t>t_star_2;
           alphahat = TT3*At1 + JJ3;
      endif;
       Phat     = TT3*Pt1*TT3' + RR3*QQ3*RR3';
    
       Phat     = .5*(Phat+Phat');
    
       /*yhat     = ZZ2*(alphahat - ssteady2) + DD2;*/
       yhat     = ZZ3*alphahat;
       nut      = YY[t,.] - yhat';
       Ft       = ZZ3*Phat*ZZ3' + HH3 + ZZ3*RR3*VV3 + (ZZ3*RR3*VV3)';

endif;
//"data";YY[t,9];
//"===========";
//"nut";nut;k34=keyw;
   Ft       = .5*(Ft+Ft');

   /** to make likelihood comparable to VAR(tcond), condition on first tcond observations **/
  /* if t > tcond;*/
    loglh    =  loglh -0.5*cols(YY)*ln(2*pi) - 0.5*ln(det(Ft)) - 0.5*nut*inv(Ft)*nut';
   /*endif;*/
//"t";t;
//"================";
//"nut";nut;
//"=============";
//"det(ft)";det(Ft);
//"=======================";
//"inv(ft)";inv(Ft);
//"======================";
//k34=keyw;
   /** Updating **/
if t < t_star_1;
   At = alphahat + Phat*ZZ'*inv(Ft)*nut';
   Pt = Phat - (Phat*ZZ'+RR*VV)*inv(Ft)*(Phat*ZZ'+RR*VV)';
endif;

if t>=t_star_1 and t < t_star_2;
   At = alphahat + Phat*ZZ2'*inv(Ft)*nut';
   Pt = Phat - (Phat*ZZ2'+RR2*VV2)*inv(Ft)*(Phat*ZZ2'+RR2*VV2)';
endif;
if t>=t_star_2;
At = alphahat + Phat*ZZ2'*inv(Ft)*nut';
   Pt = Phat - (Phat*ZZ3'+RR3*VV3)*inv(Ft)*(Phat*ZZ3'+RR3*VV3)';
endif;
   obserror[t,.] = nut;
   obsvar[t,.]   = diag(Ft)';

   t = t+1;
endo;

eofeval:

if ismiss(real(loglh))==1;
   loglh = loglhzero;
endif;

retp(real(loglh),retcode,obserror,obsvar);
endp;


