/******************************************************************
*   P_PARAMOM.G
*   Computes posterior mean and std and CI based on
*   output of posterior simulator
*
*   updated: 04/20/2006
*******************************************************************/


/********************************************************
* Load Parameter Draws from MH Output
********************************************************/
open fhdraw = ^ldraw for read;

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);


/********************************************************
* Part 1: Compute the mean of x(i) and x^2(i)
********************************************************/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
ndraws  = 0;
"===============================";
"Part 1: Moments";
"===============================";
do until eofloop; 

   drawblock  = readr( fhdraw, block_size );

   drawmean   = drawmean    + sumc(drawblock)';
   drawsqmean = drawsqmean  + sumc(drawblock^2)';
   drawcross  = drawcross   + drawblock'*drawblock;  

   ndraws = ndraws + rows(drawblock);

   "Draws " ndraws;
   ""; pause(1e-20);

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);
drawcov    = drawcross/ndraws - drawmean'*drawmean;

/* Report Posterior Mean and stderror
**
*/
"MODEL       " lmodel;
"PRIOR       " lprior;
"MH-Run      " mhrun;
"Subsample   " subT;
"Dataset     " dataselstr;

"=============================================";
" " ;

"Parameter     | Mean  |  StdD |";
outmat = para_names'~ drawmean' ~ drawstdd' ;
let mask[1,3] = 0 1 1;
let fmt[3,3] =
   "-*.*s " 12 8
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,mask,fmt);


/********************************************************
* Part 2: HPD Interval
********************************************************/
drawrow     = seekr( fhdraw, nburn); 
drawci      = zeros(2,drawdim);

"===============================";
"Part 2: HPD Interval";
"===============================";
j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, block_size );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);

      "ndraws" ndraws;
      "Column" j;
      ""; pause(1e-20);
         
   endo;
   
   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;

/* Report Posterior Mean and stderror
**
*/
cls;
"MODEL       " lmodel;
"PRIOR       " lprior;
"MH-Run      " mhrun;
"Subsample   " subT;
"Coverage    " hpdprob;  
"Dataset     " dataselstr;
"=============================================";
" " ;

"Parameter     | Mean  |  StdD | CI(LOW) | CI(HIGH)";
outmat = para_names'~ drawmean' ~ drawstdd' ~ drawci' ;
let mask[1,5] = 0 1 1 1 1;
let fmt[5,3] =
   "-*.*s " 12 8
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,mask,fmt);

create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws; 

#include p_hpdint.g;
